@@ -57,12 +57,12 @@
 #include <QtCore/QVector>
 
 #include <QtCore/QWaitCondition>
+#include <QtCore/QLoggingCategory>
 
 #include <wayland-client.h>
 
 #include <QtWaylandClient/private/qwayland-wayland.h>
 #include <QtWaylandClient/private/qtwaylandclientglobal_p.h>
-#include <QtWaylandClient/private/qwayland-xdg-shell.h>
 #include <QtWaylandClient/private/qwaylandshm_p.h>
 
 struct wl_cursor_image;
@@ -74,15 +74,15 @@ class QSocketNotifier;
 class QPlatformScreen;
 
 namespace QtWayland {
-    class qt_shell;
-    class qt_sub_surface_extension;
     class qt_surface_extension;
     class zwp_text_input_manager_v2;
-    class xdg_shell;
+    class zxdg_output_manager_v1;
 }
 
 namespace QtWaylandClient {
 
+Q_WAYLAND_CLIENT_EXPORT Q_DECLARE_LOGGING_CATEGORY(lcQpaWayland);
+
 class QWaylandInputDevice;
 class QWaylandBuffer;
 class QWaylandScreen;
@@ -92,11 +92,10 @@ class QWaylandDataDeviceManager;
 class QWaylandTouchExtension;
 class QWaylandQtKeyExtension;
 class QWaylandWindow;
-class QWaylandEventThread;
 class QWaylandIntegration;
 class QWaylandHardwareIntegration;
-class QWaylandXdgShell;
 class QWaylandShellSurface;
+class QWaylandCursorTheme;
 
 typedef void (*RegistryListener)(void *data,
                                  struct wl_registry *registry,
@@ -109,7 +108,7 @@ class Q_WAYLAND_CLIENT_EXPORT QWaylandDisplay : public QObject, public QtWayland
 
 public:
     QWaylandDisplay(QWaylandIntegration *waylandIntegration);
-    ~QWaylandDisplay(void);
+    ~QWaylandDisplay(void) override;
 
     QList<QWaylandScreen *> screens() const { return mScreens; }
 
@@ -124,8 +123,9 @@ public:
 
     QWaylandWindowManagerIntegration *windowManagerIntegration() const;
 #if QT_CONFIG(cursor)
-    void setCursor(struct wl_buffer *buffer, struct wl_cursor_image *image);
-    void setCursor(const QSharedPointer<QWaylandBuffer> &buffer, const QPoint &hotSpot);
+    void setCursor(struct wl_buffer *buffer, struct wl_cursor_image *image, qreal dpr);
+    void setCursor(const QSharedPointer<QWaylandBuffer> &buffer, const QPoint &hotSpot, qreal dpr);
+    QWaylandCursorTheme *loadCursorTheme(qreal devicePixelRatio);
 #endif
     struct wl_display *wl_display() const { return mDisplay; }
     struct ::wl_registry *wl_registry() { return object(); }
@@ -144,12 +144,14 @@ public:
     QWaylandTouchExtension *touchExtension() const { return mTouchExtension.data(); }
     QtWayland::zwp_text_input_manager_v2 *textInputManager() const { return mTextInputManager.data(); }
     QWaylandHardwareIntegration *hardwareIntegration() const { return mHardwareIntegration.data(); }
+    QtWayland::zxdg_output_manager_v1 *xdgOutputManager() const { return mXdgOutputManager.data(); }
+
 
     struct RegistryGlobal {
         uint32_t id;
         QString interface;
         uint32_t version;
-        struct ::wl_registry *registry;
+        struct ::wl_registry *registry = nullptr;
         RegistryGlobal(uint32_t id_, const QString &interface_, uint32_t version_, struct ::wl_registry *registry_)
             : id(id_), interface(interface_), version(version_), registry(registry_) { }
     };
@@ -159,6 +161,7 @@ public:
     /* wl_registry_add_listener does not add but rather sets a listener, so this function is used
      * to enable many listeners at once. */
     void addRegistryListener(RegistryListener listener, void *data);
+    void removeListener(RegistryListener listener, void *data);
 
     QWaylandShm *shm() const { return mShm.data(); }
 
@@ -173,6 +176,7 @@ public:
     QWaylandWindow *lastInputWindow() const;
     void setLastInputDevice(QWaylandInputDevice *device, uint32_t serial, QWaylandWindow *window);
 
+    bool isWindowActivated(const QWaylandWindow *window);
     void handleWindowActivated(QWaylandWindow *window);
     void handleWindowDeactivated(QWaylandWindow *window);
     void handleKeyboardFocusChanged(QWaylandInputDevice *inputDevice);
@@ -191,17 +195,25 @@ private:
     void requestWaylandSync();
 
     struct Listener {
-        RegistryListener listener;
-        void *data;
+        Listener() = default;
+        Listener(RegistryListener incomingListener,
+                 void* incomingData)
+        : listener(incomingListener), data(incomingData)
+        {}
+        RegistryListener listener = nullptr;
+        void *data = nullptr;
     };
 
-    struct wl_display *mDisplay;
+    struct wl_display *mDisplay = nullptr;
     QtWayland::wl_compositor mCompositor;
     QScopedPointer<QWaylandShm> mShm;
     QList<QWaylandScreen *> mScreens;
     QList<QWaylandInputDevice *> mInputDevices;
     QList<Listener> mRegistryListeners;
-    QWaylandIntegration *mWaylandIntegration;
+    QWaylandIntegration *mWaylandIntegration = nullptr;
+#if QT_CONFIG(cursor)
+    QMap<int, QWaylandCursorTheme *> mCursorThemesBySize;
+#endif
 #if QT_CONFIG(wayland_datadevice)
     QScopedPointer<QWaylandDataDeviceManager> mDndSelectionHandler;
 #endif
@@ -212,26 +224,22 @@ private:
     QScopedPointer<QWaylandWindowManagerIntegration> mWindowManagerIntegration;
     QScopedPointer<QtWayland::zwp_text_input_manager_v2> mTextInputManager;
     QScopedPointer<QWaylandHardwareIntegration> mHardwareIntegration;
-    QSocketNotifier *mReadNotifier;
+    QScopedPointer<QtWayland::zxdg_output_manager_v1> mXdgOutputManager;
+    QSocketNotifier *mReadNotifier = nullptr;
     int mFd;
     int mWritableNotificationFd;
     QList<RegistryGlobal> mGlobals;
     int mCompositorVersion;
-    uint32_t mLastInputSerial;
-    QWaylandInputDevice *mLastInputDevice;
+    uint32_t mLastInputSerial = 0;
+    QWaylandInputDevice *mLastInputDevice = nullptr;
     QPointer<QWaylandWindow> mLastInputWindow;
     QPointer<QWaylandWindow> mLastKeyboardFocus;
     QVector<QWaylandWindow *> mActiveWindows;
-    struct wl_callback *mSyncCallback;
+    struct wl_callback *mSyncCallback = nullptr;
     static const wl_callback_listener syncCallbackListener;
 
     void registry_global(uint32_t id, const QString &interface, uint32_t version) override;
     void registry_global_remove(uint32_t id) override;
-
-    static void shellHandleConfigure(void *data, struct wl_shell *shell,
-                                     uint32_t time, uint32_t edges,
-                                     struct wl_surface *surface,
-                                     int32_t width, int32_t height);
 };
 
 }
